#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# Disassociates a raw image file from any loopback devices
# and compresses it.
#
# The "xz" compressor is used and the original image file
# is removed unless you pass the "-k" option to this script.
#
# The environment variable IMAGE_FILE must be set and should
# point to the image file to create.
#
# Usage:
#   IMAGE_FILE="../../images/disk.img" sh disk_pack.sh [-k] [-s]
#
# Where:
#   -k  Keep the original image file after compression (otherwise it is deleted).
#   -s (script) option avoids almost all prompts
#

KEEP_OPTION=
SCRIPT_OPTION=
REMOVE_MSG="Yes"

#
# Check command line arguments
#
for ARG in $@; do
    if [ "$ARG" = "-k" ]; then
        KEEP_OPTION="$ARG"
        REMOVE_MSG="No"
    elif [ "$ARG" = "-s" ]; then
        SCRIPT_OPTION="$ARG"
    fi
done

#
# source functions
#
for NAME in "common.sh" "images.sh"; do
    NAME="functions/$NAME"
    if [ -f $NAME ]; then
        . "$NAME"
    else
        echo "\nCannot locate script file \"$NAME\". Exiting...\n"
        exit 1
    fi
done

# Check that the user is root
check_root

# Check that IMAGE_FILE is set.
image_file || quit

XZ_FILE="$IMAGE_FILE.xz"

if [ -z "$SCRIPT_OPTION" ]; then
    clear
    cat <<END

This script will disassociate a raw image file from any loopback
devices it may be connected to and compresses it to provide a
smaller file for distribution.

The "xz" compressor is used and the original image file is removed
unless you call this script with the "-k" (keep) option. The "xz"
compressor creates files that are about 20% smaller than "bipzip2 -9".

The "-T0" option is used with "xz" to use all available cores.

           IMAGE FILE: $IMAGE_FILE
          OUTPUT FILE: $XZ_FILE
    REMOVE IMAGE FILE: $REMOVE_MSG
END
    prompt_enter
else 
    cat <<END

Compressing raw image file with "xz" compressor.

           IMAGE FILE: $IMAGE_FILE
          OUTPUT FILE: $XZ_FILE
    REMOVE IMAGE FILE: $REMOVE_MSG

END
fi

# check that file pointed to by IMAGE_FILE exists
image_exists || quit

# disconnect image file from any loop devices
unloop || quit

if [ -e "$XZ_FILE" ]; then
    echo "Compressed file \"$XZ_FILE\", already exists, deleting\n"
    rm -f "$XZ_FILE" || quit
fi

CMD="xz -T0 $KEEP_OPTION $IMAGE_FILE"
echo "Compressing raw image file with the following command:\n\n\t$CMD\n"
if $CMD; then
    :
else
    script_error $? "$CMD"
    quit
fi

exit 0
