#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# Common script functions.
#

#
# Check for required environment variables and calls
# "exit 1" if any values are not defined.
#
# Usage: check_env NAME1, NAME2...
#
check_env() {
    LOCAL NAME
    for NAME in $*; do
        eval VALUE="\$$NAME"
        if [ -z "$VALUE" ]; then
            echo "\nEnvironment variable \"$NAME\" is not set.\n"
            exit 1
        fi
    done
}

#
# Check that the user is "root".
#
check_root() {
    case $(whoami) in
        (root)  ;;
        (*) echo "\nYou must be root to run this script.\n"
            exit 1
            ;;
    esac
}

#
# Prompt user with restricted list of response strings.
#
# Usage: prompt_choice PROMPT_STRING RESPONSE1 RESPONSE2
#
# Result is left in $ANS.
#
prompt_choice() {
    local PROMPT="$1"
    local i
    shift
    echo
    while : ; do
        echo -n "$PROMPT"
        read ANS
        for i in $@; do
            if [ "X$ANS" = "X$i" ]; then
                return
            fi
        done
        echo "\nPlease input one of: $@\n"
    done
}

#
# Prompt user to press ENTER.
#
# Usage: prompt_enter
#
# Result is left in $ANS.
#
prompt_enter() {
    local INPUT
    echo -n "\nPress ENTER to continue: "
    read INPUT
}

#
# Prompt user for a "y" response or quit.
#
# Usage: prompt_y_or_quit
#
prompt_y_or_quit() {
    local ANS
    echo -n "\nInput \"y\" to continue or anything else to quit: "
    read ANS
    if [ "X$ANS" != "Xy" ]; then
        echo "\nExiting...\n"
        exit 1
    fi
}

#
# Prompt user for a "y" or "n" response.
#
# Usage: prompt_yn PROMPT
#
# Result is left in $ANS.
#
prompt_yn() {
    prompt_choice "$1" y n
}

#
# Runs a script and handles errors.
#
script_run() {
    local SCRIPT="$1"
    shift
    if [ -r "$SCRIPT" ]; then
        if sh "./$SCRIPT" $@; then
            if [ -z "$SCRIPT_OPTION" ]; then
                prompt_enter
            fi
        else
            script_error $? "sh ./$SCRIPT $@"
            quit
        fi
    else
        echo "\nCannot locate \"$SCRIPT\""
        ERROR=1
        script_shell
        quit
    fi
}

#
# Called after a script returns an error with the option to
# shell out for inspection.
#
# Sets ERROR
#
script_error() {
    ERROR=$1
    shift
    local CMD="$*"
    echo "\nError $ERROR returned by by following command:\n\n\t$CMD"
    script_shell
}

#
# Offers to shell out for inspection.
#
script_shell() {
    prompt_yn "Do you want to spawn a shell to look around (y/n)? "
    if [ "$ANS" = "y" ]; then
        echo "\nSpawning /bin/sh. Type \"exit\" to return to this script.\n"
        /bin/sh
    fi
    echo "\nCleaning up..."
}

#
# Print an "Exiting..."  message and exit the program
# with "exit 1"
#
quit() {
    echo "\nExiting..."
    exit 1
}
