#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# Function related to disk images and loopback devices.
#

check_losetup() {
    if which losetup >/dev/null; then
        return 0
    else
        echo "\nThe program \"losetup\" is not installed."
        return 1
    fi
}

#
# Checks that the environment variable IMAGE_FILE is set and 
# the file it points to exists and is a regular file.
#
# Usage: image_exists
#
# Returns: 0=success, 1=error (message printed)
#
image_exists() {
    # Check that IMAGE_FILE is set.
    image_file || return 1
    if [ -e "$IMAGE_FILE" ]; then
        if [ -f "$IMAGE_FILE" ]; then
            return 0
        else 
            echo "\nImage file \"$IMAGE_FILE\" is not a regular file."
        fi
    else
        echo "\nImage file \"$IMAGE_FILE\" does not exist."
    fi
    return 1
}

#
# Checks that environment variable IMAGE_FILE is set.
#
# Usage: image_file
#
# Returns: 0=success, 1=error (message printed)
#
image_file() {
    if [ -z "$IMAGE_FILE" ]; then
        echo "\nEnvironment variable \"IMAGE_FILE\" is not set."
        return 1
    fi
    return 0
}

#
# Disociates any loopback devices assocated with IMAGE_FILE.
#
# Usage: unloop
#
unloop() {
    local FOUND
    local IMAGE_PATH
    local LOOP
    local MOUNT
    local MESSAGE
    local OIFS

    # Check that IMAGE_FILE is set.
    image_file || return 1
    check_losetup || return 1
    IMAGE_PATH=$(readlink -f $IMAGE_FILE) || return 1
    OIFS="$IFS"
    IFS=
    # next command eats last newline
    FOUND=$(losetup -a |grep $IMAGE_PATH)
    IFS="$OIFS"
    if [ -n "$FOUND" ]; then
        echo "\nImage file \"$IMAGE_FILE\" is in-use as follows:\n\n$FOUND"
        echo "$FOUND" | while read LINE; do
            echo "\nProcessing: $LINE"
            IFS=':'
            set $LINE
            IFS="$OIFS"
            LOOP="$1"
            if MOUNT=$(mount |grep "$LOOP"); then
                echo "Device \"$LOOP\" is mounted, unmounting"
                if umount $1; then
                    echo "Unmounted device \"$LOOP\""
                else
                    echo "Could not unmount device \"$LOOP\"\n"
                    return 1
                fi
            fi
            echo "Disconnecting device \"$LOOP\""
            if losetup -d "$LOOP"; then
                :
            else
                echo "Could not disconnect device \"$LOOP\"\n"
                return 1
            fi
        done
        MESSAGE="has been disassociated from all"
    else
        MESSAGE="is not associated with any"
    fi
    echo "\nImage file \"$IMAGE_FILE\" $MESSAGE loopback devices.\n"
    return 0
}
