#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# Functions related to target devices.
#

#
# SECTORS is needed when calculating the root partition
# size and cannot be local. This value will only be set
# if select_target() is called.
#
export SECTORS=0

#
# Validates TARGET_DEVICE and calls select_target()
# if the selected device is missing or is invalid.
#
# Usage: check_target
#
# Where TEXT is an optional string to print at the top of the
# instructions.
#
# Sets and exports TARGET_DEVICE.
# Only sets and exports LOOP_DEVICE and SECTORS if
# select_target is called.
#
check_target() {
    local MSG
    #
    # Check whether $TARGET_DEVICE is already set
    #
    if [ -z "$TARGET_DEVICE" ]; then
        MSG="TARGET_DEVICE environment variable is not set."
    else
        # Modify TARGET_DEVICE to add "/dev/", if necessary.
        case "$TARGET_DEVICE" in
            /dev/*);;
            *) TARGET_DEVICE="/dev/$TARGET_DEVICE";;
        esac

        #
        # Verify that the device exists and is block special
        #
        if [ -b "$TARGET_DEVICE" ]; then
            return
        fi
        MSG="TARGET_DEVICE \"$TARGET_DEVICE\""
        if [ -e "$TARGET_DEVICE" ]; then
            MSG="$MSG is not block special"
        else
            MSG="$MSG does not exist"
        fi
        TARGET_DEVICE=
    fi
    if [ -n "$MSG" ]; then
        echo "\n$MSG"
        prompt_enter
    fi
    select_target
}

#
# Selects a target device.
#
# Usage: select_target TEXT
#
# Where TEXT is an optional string to print at the top of the
# instructions.
#
# Sets and exports TARGET_DEVICE, LOOP_DEVICE and SECTORS.
#
select_target() {
    local DNAME
    local MOUNT

    clear
    cat <<END

${1}Select a target device ("sdX" or "loopN")

The target must be a block or loopback device that is at
least 2GiB in size. No parts of this device can be mounted.

EXTREMELY IMPORTANT: This script will be writing directly to
whichever device you select here. Any existing file system
and data on that device will become inaccessible.

These are the block devices found on your system:

END
    lsblk
    while :; do
        if [ -z "$TARGET_DEVICE" ]; then
            echo -n "\nInput the short name of the target device (sdX or loopN): "
            read TARGET_DEVICE
            if [ -z "$TARGET_DEVICE" ]; then
                echo "\nPlease input a device name."
                continue;
            fi
        fi
        case "$TARGET_DEVICE" in
            (sd[a-z]|loop[0-9]|loop[0-9][0-9])  # OK
                DNAME="/dev/$TARGET_DEVICE"
                MOUNT=$(mount | grep $DNAME)
                if [ -n "$MOUNT" ]; then
                    echo "\nBlock device \"$TARGET_DEVICE\" has active mounts:\n$MOUNT"
                    TARGET_DEVICE=""
                    continue;
                fi
                DNAME="/sys/class/block/$TARGET_DEVICE"
                if [ ! -e "$DNAME" ]; then
                    echo "\nBlock device \"$TARGET_DEVICE\" does not exist."
                    TARGET_DEVICE=""
                    continue;
                fi
                #
                # Make sure the device is at least 2 GiB or 4,194,304 512-byte sectors.
                #
                # 1 MiB is 2,048 sectors
                # 1 GiB is 2,097,152 sectors
                #
                # /sys/class/block/sdX/size is used which returns the number
                # of sectors in the block device.
                #
                # Because this script has to be able to run on a Kirkwood device
                # (which has 32 bit shell integers, thus 2 GiB max int or a 1TB max
                # disk size), regular shell math won't work for multi-terrabyte
                # sized disks. As a result, string length is used to check for
                # very large block counts and integer comparison is only used
                # when the string length of the disk size is less than eight
                # digits (or 9,999,999 sectors).
                #
                SECTORS=$(cat "$DNAME/size")
                #echo "BLOCK DEVICE HAS $SECTORS SECTORS"
                #echo "LENGTH of SECTORS STRING: ${#SECTORS}"
                if [ ${#SECTORS} -lt 8 -a $SECTORS -lt 4194304 ]; then
                    echo "\nBlock device \"$TARGET_DEVICE\" is less than 2GiB. Please choose another."
                    TARGET_DEVICE=""
                    continue;
                fi
                echo "\nSelected target device: $TARGET_DEVICE"
                prompt_yn "Are your absolutely sure (y/n)? "
                if [ "$ANS" = "y" ]; then
                    break
                fi
                ;;
            *)
                echo "\nThe device name should be \"sdX\" or \"loopN\""
                echo "where the \"X\" is a lower case letter and \"N\" is a number.\n"
                echo "You entered: \"$TARGET_DEVICE\"."
        esac
        TARGET_DEVICE=""
    done
    case "$TARGET_DEVICE" in
        (sd*)   LOOP_DEVICE=;;
        (*) LOOP_DEVICE="$TARGET_DEVICE";;
    esac
    export TARGET_DEVICE="/dev/$TARGET_DEVICE"
    export LOOP_DEVICE
}
