#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# Creates and installs a Kirkwood kernel, initial RAM
# disk and root file system onto a target device.
#

#
# Usage:
#
# sh install.sh [-b] [-s]
#
# Where:
#   -b (build) causes this script to call disk_pack.sh after it is done.
#   -s (script) option avoids almost all prompts
#

BUILD_OPTION=
SCRIPT_OPTION=

#
# Configure the script.
#
config() {

    #
    # Check the current directory
    #
    if [ ! -d ../dts ]; then
        cat <<END

You appear to be in the wrong directory.
Cannot locate "../dts" directory.
You must run this script from the "boot/tools" directory.
END
        quit
    fi

    #
    # Source functions
    #
    for NAME in "common.sh" "dts.sh" "kernel_version.sh" "target.sh" "tools_version.sh"; do
        NAME="functions/$NAME"
        if [ -f $NAME ]; then
            . "$NAME"
        else
            echo "\nCannot locate script file \"$NAME\". Exiting...\n"
            exit 1
        fi
    done

    # Check that the user is root
    check_root

    #
    # Make sure kernel version is set
    #
    if [ -z "$KERNEL_VERSION" ]; then
        echo "\nKERNEL_VERSION is not set.\nIt should be found in kernel_version.sh"
        quit
    fi
}

help() {
    clear
    cat <<END

This script will install Kirkwood Debian Linux version $KERNEL_VERSION
on an IDE drive or USB flash stick or disk. If the version number
is incorrect, edit the file "kernel_version.sh" and start over.

The only difference between the IDE and USB versions is what
the root file system and swap space are called. By using two
different sets of names (rootfs + swap or usbroot + usbswap)
the USB and IDE versions can coexist on the same Kirkwood device.

This allows you to create a USB rescue stick (or disk) that can
be plugged into the USB port of the IX2 and booted in order to
mount the Kirkwood IDE drive(s) and perform whatever is needed
to get the system running again.

The USB installation media can also be used to boot Debian on
your Kirkwood machine and then run the installation script from
the USB device. In this way, you can install Debian on the IDE
drive when you don't have an external hard disk dock or you
don't want to install the target IDE drive on your desktop.

Older versions of U-Boot (such as on the Lenovo IX2) have
problems booting from a USB disk. It seems to be an issue
with disk spin-up time. External USB SDD devices work fine.
END
    prompt_enter
    clear
    cat <<END

If you will be creating a USB stick, make sure to use a quality
device (like PNY). Cheap USB sticks (like the kind you get for
free) can waste hours of your time and leave you with an
unreliable system.

Also note that running Linux on a flash device needs careful
design decisions to prevent wearing out the device. OpenWrt
has been designed for this purpose and minimizes updates to
the flash memory. If you want a flash-based O/S, consider
using OpenWrt. Kirkwood Debian should be run on a hard drive
or an SSD which is designed as a hard disk replacement.

The intent behind the USB stick installation is to create
a runnable Debian O/S that can be used as a rescue or recovery
device. The installation incudes a swap partition to give you
more virtual memory but this will also wear out a flash device.
Booting from a USB stick is not intented for long-term use.
END
    prompt_enter
    clear
    cat <<END

This script was originally written to work around the problem of
how to boot a GUID Partition Table (GPT) using older versions of
U-Boot which only understands fdisk partition tables. It also
supports newer U-Boot versions that don't need this work-around.

This script should work with any Kirkwood board (with older or
newer U-Boot) provided the board has a ".dtb" (device tree) file
in the "boot/dts" directory. For older U-Boot versions, this
script creates a Debian kernel with the DTS file appended which
results in a kernel that will only run on that board.

For newer U-Boot versions, the DTS file is not appended to the
kernel and the GPT work-around is not used. The initial RAM disk
is generic and should work with any Kirkwood board supported by
the kernel.

If you have an old U-Boot version (as found in the Lenovo IX2),
you need to know which DTS file to use. For the Lenovo IX2, two
common versions are the "ix2-dl" and the "ix2-ng".  The "ix2-dl"
has 1 GiB of NAND flash and the "ix2-ng" has 1 MiB of SPI flash.
The only difference is how the flash is mapped by the kernel.
This is particularly important if you want to run "fw_printenv"
or "fw_saveenv".

The instalation script will provide the opportunity to select
from any of the DTS files in "boot/dts".

For additional information, load the HTML file "ix2.html"
into a web browser.
END
    prompt_enter
    clear
    cat <<END

A full installation includes the following steps:

1) Select a target device ("sdX" or "loopN").
2) Select an installation type ("usb" or "ide").
3) Select a DTS file.
4) Select swap and root partion sizes.
5) Partition the target device with parted.
6) Prepare the swap partition and format the root partition.
7) Create the kernel and initial RAM disk files.
8) Install the kernel and initial RAM disk file on the target device.
9) Copy the root file system to the target device and customize.

You can avoid having to type in values for steps 1-4 and 9 by setting
the following environment variables before you call this script:

TARGET_DEVICE: "sdX" or "loopN"
 INSTALL_TYPE: "usb" or "ide"
  DEVICE_TYPE: "all" or shortened DTS name
    SWAP_SIZE: 256, 512, 768 or 1024 (MiB)
    ROOT_SIZE: -1=all, or 2-512 (GiB)
       FSTYPE: "ext2", "ext3" or "ext4"
       CUSTOM: "orig" or "ix2"

See the file "ix2_ng_stick.sh" for an example of this usage.
END
    prompt_enter
    clear
    cat <<END

A DEVICE_TYPE of "all" is used for new U-Boot versions. Otherwise it
is a DTS file name without the leading "kirkwood-" or ending ".dtb".
For example: "lenovo-ix2-dl" or "lenovo-ix2-ng".

FSTYPE is used in steps 5 and 6 and is the file system type to
use with the root file system (by default, ext3 is used for USB
sticks and ext4 for hard disks).

Steps 5-9 are optional in the sense that they can be skipped
if they have already been done. Each step must be done at least
once but you can exit the script at any time and start over,
if required.

Step 9 copies file to create the new root file system and then
runs a customization script. This script copies modified files
from a directory under "boot/tools/custom" (such as "orig" or
"ix2"). Each directory contains an "install.sh" script to
perform the customization. If you have an IX2 device, you must
select the "ix2" customization script for your system to work
correctly. You can pass the name of the customzation to use
in the "CUSTOM" environment variable.
END
    prompt_enter
    clear
    cat <<END

The provided kernel initial RAM disk only supports fsck for ext2,
ext3, ext4 and nfs. This means the root file system needs to be
one of these types. If you want to use something else, one solution
is to allocate a small root parition (like 10 GiB) using ext* and
formatting the rest of the space with something else. Then
add that something else to /etc/fstab so it gets mounted once
the kernel has access to all of the modules and packages in the
root file system that support your something else.

VERY IMPORTANT: If you have an older version of U-Boot, you must
perform a one-time change to the U-Boot environment as documented
in the "ix2.html" help page. There is a convenience script called
"set_env.sh" that can be run on the stock O/S of your Kirkwood
device to set the appropriate environment values. This only has
to be done once.

Once the environment has been changed you won't be able to boot
the stock O/S that came with your Kirkwood board without changing
the "bootcmd" environment variable back to its original value.
END
    prompt_enter
}

#
# Display introduction screen.
#
intro() {
    clear
    cat <<END

This script will install Kirkwood Debian Linux version $KERNEL_VERSION
on an IDE drive or USB flash stick or disk. If the version number
is incorrect, edit the file "kernel_version.sh" and start over.

    Tools version: $TOOLS_VERSION

*** NO WARRANTY ***
This file and all associated software is provided "as is",
without warranty of any kind, express or implied, including but not
limited to the warranties of merchantability, fitness for a
particular purpose and noninfringement. In no event shall the
author be liable for any claim, damages or other liability, whether
in an action of contract, tort or otherwise, arising from, out of
or in connection with this file and associated software or the
use or other dealings in same.
END

    #
    # Display found environment variables.
    #
    OUTPUT=
    for i in TARGET_DEVICE INSTALL_TYPE TARGET_TYPE DEVICE_TYPE SWAP_SIZE ROOT_SIZE FSTYPE CUSTOM; do
        eval VALUE="\$$i"
        if [ -n "$VALUE" ]; then
            OUTPUT="${OUTPUT}$i: $VALUE\n"
        fi
    done
    if [ -n "$OUTPUT" ]; then
        echo "\nThe following environment variables were found:\n"
        echo -n "$OUTPUT"
    fi

    if [ -z "$SCRIPT_OPTION" ]; then
        prompt_yn "Do you want to see the instructions (y/n)? "
        if [ "$ANS" = "y" ]; then
            help
        fi
    else
        prompt_enter
    fi
}

#
# Main function, runs all steps.
#
main() {
    local STEP
    # Configure script
    config
    # Display intro screen
    intro
    # Run all steps
    for STEP in 1 2 3 4 5 6 7 8 9; do
        eval "step$STEP"
    done
    echo "\nInstallation complete.\n"
    if [ -n "$BUILD_OPTION" ]; then
        CMD="sh ./disk_pack.sh $SCRIPT_OPTION"
        echo "Compressing resulting image file with the following command:\n\n\t$CMD\n"
        if $CMD; then
            :
        else
            script_error $? "$CMD"
            quit
        fi
    fi
}

#
# Print the current configuration.
#
print_config() {
    echo "\n Target device: $TARGET_DEVICE"
    if [ -n "$DEV_SWAP" ]; then
        echo "Swap partition: $DEV_SWAP"
    fi
    if [ -n "$DEV_ROOT" ]; then
        echo "Root partition: $DEV_ROOT"
    fi
    cat << END
  Install type: $INSTALL_TYPE
   Target type: $TARGET_TYPE
   Device type: $DEVICE_TYPE
     Swap size: $SWAP_SIZE MiB
     Root size: $ROOT_SIZE GiB
   File system: $FSTYPE
END
}

#
# Prompt for a file system type.
#
prompt_fstype() {
    if [ -z "$HAS_FSTYPE" ]; then
        HAS_FSTYPE="Y"      # only prompt once
        case "$FSTYPE" in
            ('' | ext[2-4]) ;; # OK
            (*)
                echo "\nInvalid FSTYPE: \"$FSTYPE\""
                prompt_enter
                FSTYPE=""
                ;;
        esac
        if [ -z "$FSTYPE" ]; then
            local WHAT
            if [ "$TARGET_TYPE" = "disk" ]; then
                WHAT="disk"
                FSTYPE="ext4"
            else
                WHAT="USB stick"
                FSTYPE="ext3"
            fi
            cat <<END

Because the target device is a $WHAT, the suggested
root file system type is $FSTYPE. This script supports
ext2, ext3 and ext4 file systems. These are also the
types supported by fsck on the initial RAM disk.
END
            prompt_yn "Do you want to use $FSTYPE (y/n)? "
            if [ "$ANS" = "n" ]; then
                prompt_choice "Which ext file system should be used (ext2/ext3/ext4)? " ext2 ext3 ext4
                FSTYPE="$ANS"
            fi
        fi
        export FSTYPE
        echo "\nSelected root file system type: $FSTYPE"
        if [ -z "$SCRIPT_OPTION" ]; then
            prompt_enter
        else
            echo
        fi
    fi
}

select_custom() {
    if [ -n "$CUSTOM" ]; then
        if [ ! -d "custom/$CUSTOM" ]; then
            echo "\nNo such custom directory: "boot/tools/custom/$CUSTOM"\n"
            CUSTOM=""
        elif [ -n "$SCRIPT_OPTION" ]; then
            echo "\nSelected customizer: $CUSTOM\n"
            return
        fi
    fi
    clear
    cat <<END

--> Select Customized Root File System

Because of variations in target devices, some of the files in
the root file system must be customized. Examples include:

    etc/ntp.conf
    etc/initramfs-tools/initramfs.conf
    etc/rc.local
    etc/fstab
    etc/fw_env.config

Each directory under "boot/tools/custom" contains a supported
customization. Currently, "ix2" and "orig" are implemented.
"orig" installs the original "Bohdi" version of these files.

If your device is an IX2, please select "ix2".

The name of the custom directory can be passed in the
environment variable "CUSTOM", as in "CUSTOM=ix2".

END
    if [ -z "$CUSTOM" ]; then
        prompt_choice "Which customizer do you want to use (ix2/orig)? " ix2 orig
        CUSTOM="$ANS"
    fi
    if [ ! -d "custom/$CUSTOM" ]; then
        echo "\nNo such directory: "boot/tools/custom/$CUSTOM"."
        quit
    fi
    if [ ! -f "custom/$CUSTOM/install.sh" ]; then
        echo "\nNo such file: "boot/tools/custom/$CUSTOM/install.sh"."
        quit
    fi
    export CUSTOM
    echo "\nSelected customizer: $CUSTOM"
    prompt_enter
}

#
# Step 1
#
# Sets and exports TARGET_DEVICE, LOOP_DEVICE and SECTORS.
#
# This step is always run, even when running in script mode (SCRIPT_OPTION)
#
step1() {
    select_target "--> Step 1: "
}

step2() {
    if [ -n "$INSTALL_TYPE" ]; then
        case "$INSTALL_TYPE" in
            (ide|usb) return ;; # OK
        esac
        echo "\nInvalid INSTALL_TYPE: \"$INSTALL_TYPE\""
        INSTALL_TYPE=
    fi
    clear
    cat << END

--> Step 2: Select an installation type ("usb" or "ide")

Target device: $TARGET_DEVICE

Selecting "usb" means that you intend to run this copy of Debian
from the USB port on your IX2. This can either be a USB stick or
an external USB drive. If you select "usb" you will then be asked
what type of device will be plugged into the USB port (stick or drive).

Selecting "ide" means that you intend to run this copy of Debian
from one of the IDE drive bays in your IX2.
END
    prompt_choice "Create a USB (u) or IDE (i) install? " u i
    if [ "$ANS" = "u" ]; then
        INSTALL_TYPE="usb"
    else
        INSTALL_TYPE="ide"
    fi
    export INSTALL_TYPE
    echo "\nSelected install type: $INSTALL_TYPE"
    if [ "$INSTALL_TYPE" = "usb" ]; then
        case "$TARGET_TYPE" in
            ''|disk|stick) ;; # OK
            *)
                echo "\nInvalid TARGET_TYPE: \"$TARGET_TYPE\""
                TARGET_TYPE=""
                ;;
        esac
        if [ -z "$TARGET_TYPE" ]; then
            prompt_choice "Is your USB device a flash stick (s) or hard disk (d)? " s d
            if [ "$ANS" = "s" ]; then
                TARGET_TYPE="stick"
            else
                TARGET_TYPE="disk"
            fi
        fi
    else
        TARGET_TYPE="disk"
    fi
    export TARGET_TYPE
    echo "\nSelected target type: $TARGET_TYPE"
    prompt_enter
}

step3() {
    #
    # Set SWAP_LABEL and ROOT_LABEL based on INSTALL_TYPE from step2
    #
    if [ $INSTALL_TYPE = "ide" ]; then
        SWAP_LABEL="swap"
        ROOT_LABEL="rootfs"
    else
        SWAP_LABEL="usbswap"
        ROOT_LABEL="usbroot"
    fi
    if [ -n "$DEVICE_TYPE" ]; then
        dts_validate
        if [ -n "$DEVICE_TYPE" ]; then
            return
        fi
    fi
    clear
    cat << END

--> Step 3: Select a DTS file

Target device: $TARGET_DEVICE
 Install type: $INSTALL_TYPE
  Target type: $TARGET_TYPE
END
    dts_prompt
}

#
# This function sets the value of SWAP_SIZE and ROOT_SIZE.
#
# 1 GiB is 2,097,152 sectors
#
# This script has to be able to run on a Kirkwood device (which has 32
# bit shell integers, thus 2GiB max int or a 1TB max disk size). In order
# to calculate the ending root partion sector number, this number must
# be 1 TB or less. Since this doesn't include the swap partition, the
# maximum root partition size used here is 512 GiB because, really.
#
step4() {
    local OK=0
    case "$SWAP_SIZE" in
        ('') ;;
        (256|512|768|1024)
            OK=1
            ;;
        (*)
            echo "\nInvalid swap size: $SWAP_SIZE"
            SWAP_SIZE=
            ;;
    esac
    if [ -n "$ROOT_SIZE" ]; then
        validate_root_size
        if [ -n "$ROOT_SIZE" ]; then
            OK=$(expr $OK + 1)
        fi

    fi
    if [ $OK -eq 2 ]; then
        return
    fi
    clear
    cat << END

--> Step 4: Select swap and root partion sizes

Target device: $TARGET_DEVICE
 Install type: $INSTALL_TYPE
  Target type: $TARGET_TYPE
  Device type: $DEVICE_TYPE

This script always creates a swap partition. The default size
is 768 MiB but you can choose 256, 512, 768 or 1024 MiB.

The default size for the root partition is the remaining space
on the device. Alternately, you can choose a size from 2-512 GiB.
END
    if [ -z "$SWAP_SIZE" ]; then
        prompt_yn "Is a swap size of 768 MiB OK (y/n)? "
        if [ "$ANS" = "y" ]; then
            SWAP_SIZE="768"
        else
            prompt_choice "Input a swap size (256/512/768/1024): " 256 512 768 1024
            SWAP_SIZE="$ANS"
        fi
        export SWAP_SIZE
    else 
        echo "\nUsing swap size of $SWAP_SIZE MiB, passed in via SWAP_SIZE"
    fi
    if [ -z "$ROOT_SIZE" ]; then
        prompt_yn "Should the root partition fill the remaining space (y/n)? "
        if [ "$ANS" = "y" ]; then
            export ROOT_SIZE="-1"
            return
        fi
        while :; do
            echo -n "\nInput the root partion size in GiB (2-512): "
            read ROOT_SIZE
            validate_root_size
            if [ -n "$ROOT_SIZE" ]; then
                break
            fi
        done
        export ROOT_SIZE
    else
        echo "\nUsing root size of $ROOT_SIZE GiB, passed in via ROOT_SIZE"
    fi
}

step5() {
    local CMD1
    local CMD2
    local PARTED
    local ROOT_NAME
    local ROOT_START
    local ROOT_END
    local SWAP_NAME
    local SWAP_START
    local SWAP_END
    local SECTORS
    if [ -z "$SCRIPT_OPTION" ]; then
        clear
        cat << END

--> Step 5: Partition the target device with parted

Target device: $TARGET_DEVICE
 Install type: $INSTALL_TYPE
  Target type: $TARGET_TYPE
  Device type: $DEVICE_TYPE
    Swap size: $SWAP_SIZE MiB
    Root size: $ROOT_SIZE GiB

This step will erase any existing partition table on $TARGET_DEVICE.
It will then create a new GUID Partition Table with a $SWAP_SIZE MiB
END
        if [ "$ROOT_SIZE" = "-1" ]; then
            echo "swap partition and a root partition which uses the rest of the space."
        else
            echo "swap partition and a $ROOT_SIZE GiB root partition."
        fi
        prompt_yn "Do you want to partition the target device (y/n)? "
        if [ "$ANS" = "n" ]; then
            echo "\nSkipped making partition table"
            prompt_enter
            return
        fi
    else
        echo "\n--> Step 5: Partition the target device with parted\n"
    fi
    PARTED=$(which parted)
    if [ -z "$PARTED" ]; then
        echo "Please install parted (the GPT partition manager)"
        quit
    fi
    prompt_fstype
    if [ "$INSTALL_TYPE" = "ide" ]; then
        SWAP_NAME="swap"
        ROOT_NAME="rootfs"
    else
        SWAP_NAME="usbswap"
        ROOT_NAME="usbroot"
    fi
    if [ "$DEVICE_TYPE" = "all" ]; then
        #
        # New U-Boot:
        # No kernel or initrd before swap
        # swap starts at 2048
        # This is at the first 1 MiB
        #
        SWAP_START=2048
    else
        #
        # Old U-Boot:
        # kernel starts at 2048 (size of 32768)
        # initrd starts at 34816 (size of 32768)
        # swap starts at 67584
        SWAP_START=67584
    fi
    #
    # Number of sectors in swap and root partitions.
    #
    # 1 MiB is 2048 sectors.
    # 1 GiB is 2097152 sectors.
    #
    # Partitions should be aligned on 1MiB.
    #
    SECTORS=$(expr $SWAP_SIZE \* 2048)
    SWAP_END=$(expr $SWAP_START + $SECTORS - 1)
    ROOT_START=$(expr $SWAP_END + 1)
    if [ "$ROOT_SIZE" = "-1" ]; then
        ROOT_END="-1"
    else
        SECTORS=$(expr $ROOT_SIZE \* 2097152)
        ROOT_END=$(expr $ROOT_START + $SECTORS - 1)
    fi
    #
    # parted options used here:
    #   unit s - Units are in sectors
    #   mklabel gtp - Create a GTP partition table
    #
    CMD1="dd if=/dev/zero of=$TARGET_DEVICE bs=512 count=1"
    #CMD1="wipefs -a $TARGET_DEVICE"
    CMD2="parted -a opt -- $TARGET_DEVICE unit s  mklabel gpt  mkpart $SWAP_LABEL linux-swap $SWAP_START $SWAP_END \
 mkpart $ROOT_LABEL $FSTYPE $ROOT_START $ROOT_END"
    if [ -z "$SCRIPT_OPTION" ]; then
        clear
        cat <<END

--> Partition $TARGET_DEVICE

The following partitions will be created. The values for
"start" and "end" are expressed in sectors of 512 bytes.
A value of -1 uses the remaining space on the device.

Partition 1:
    label: $SWAP_NAME
    type : linux-swap
    start: $SWAP_START
    end  : $SWAP_END
    size : $SWAP_SIZE MiB

Partition 2:
    label: $ROOT_NAME
    type : $FSTYPE
    start: $ROOT_START
    end  : $ROOT_END
        size : $ROOT_SIZE GiB

*** WARNING ***
You are about to overwrite any existing partitions and data on
$TARGET_DEVICE by running the following commands:

$CMD1
$CMD2

Note that parted may want to alter the partitions slightly. This
often happens when an "end" value of "-1" is used (entire device)
and the device does not end on a 1 MiB boundary.  Generally, you
can accept whatever parted suggests, as long as it seems reasonable.
END
        prompt_yn "Are your sure you want to proceed (y/n)? "
        if [ "$ANS" = "n" ]; then
            echo "\nSkipped making partition table"
            prompt_enter
            return
        fi
    fi
    echo "\nClearing existing partitions\n"
    if $CMD1; then
        echo "\nExisting partitions cleared"
    else
        script_error $? "$CMD1"
        quit
    fi
    echo "\nParitioning device\n"
    if $CMD2; then
        echo "Here is the new partition table:\n"
        parted $TARGET_DEVICE unit MiB print
    else
        script_error $? "$CMD2"
        quit
    fi
    if [ -z "$SCRIPT_OPTION" ]; then
        prompt_enter
    fi
}

step6() {
    local CMD
    local FORMAT

    if [ -z "$LOOP_DEVICE" ]; then
        DEV_SWAP="${TARGET_DEVICE}1"
        DEV_ROOT="${TARGET_DEVICE}2"
    else
        DEV_SWAP="${TARGET_DEVICE}p1"
        DEV_ROOT="${TARGET_DEVICE}p2"
    fi
    export DEV_SWAP DEV_ROOT
    if [ -z "$SCRIPT_OPTION" ]; then
        clear
        cat << END

--> Step 6: Prepare the swap partition and format the root partition

 Target device: $TARGET_DEVICE
Swap partition: $DEV_SWAP
Root partition: $DEV_ROOT
  Install type: $INSTALL_TYPE
   Target type: $TARGET_TYPE
   Device type: $DEVICE_TYPE
     Swap size: $SWAP_SIZE MiB
     Root size: $ROOT_SIZE GiB
END
        if [ -n "$HAS_FSTYPE" ]; then
            echo "  File system: $FSTYPE"
        fi
        prompt_yn "Prepare the swap partition (y/n)? "
    else
        echo "\n--> Step 6: Prepare the swap partition and format the root partition\n"
        ANS="y"
    fi
    if [ "$ANS" = "y" ]; then
        if [ ! -e $DEV_SWAP ]; then
            echo "\nSwap partition \"$DEV_SWAP\" does not exist"
            quit
        fi
        CMD="mkswap -L $SWAP_LABEL $DEV_SWAP"
        echo "\nPrepare swap partition with the following command:\n\n\t$CMD"
        if [ -z "$SCRIPT_OPTION" ]; then
            prompt_yn "Are your sure you want to proceed (y/n)? "
        else
            ANS="y"
        fi
        if [ "$ANS" = "y" ]; then
            echo
            if $CMD; then
                echo "\nSuccessfully prepared swap partition"
            else
                script_error $? "$CMD"
                quit
            fi
        else
            echo "\nSkipped preparing the swap partition"
        fi
    else
        echo "\nSkipped preparing the swap partition"
    fi

    if [ -z "$SCRIPT_OPTION" ]; then
        prompt_yn "Format the root partition (y/n)? "
    else 
        ANS="y"
    fi
    if [ "$ANS" = "y" ]; then
        prompt_fstype
        if [ ! -e $DEV_ROOT ]; then
            echo "\nRoot partition \"$DEV_ROOT\" does not exist"
            quit
        fi
        PROG="mkfs.$FSTYPE"
        if [ $FSTYPE = "ext4" ]; then
            CMD="mkfs.ext4 -E lazy_itable_init=0,lazy_journal_init=0 -L $ROOT_LABEL $DEV_ROOT"
        else
            CMD="$PROG -L $ROOT_LABEL $DEV_ROOT"
        fi
        echo "\nFormat $DEV_ROOT as $FSTYPE with the following command:\n\n\t$CMD"
        if [ -z "$SCRIPT_OPTION" ]; then
            prompt_yn "Are your sure you want to proceed (y/n)? "
        else
            ANS="y"
        fi
        if [ "$ANS" = "y" ]; then
            echo
            if $CMD; then
                :
                echo "\nSuccessfully formatted root partition"
            else
                script_error $? "$CMD"
                quit
            fi
        else
            echo "\nSkipped formatting the root partition"
        fi
    else
        echo "\nSkipped formatting the root partition"
    fi
    if [ -z "$SCRIPT_OPTION" ]; then
        prompt_enter
    fi
}

step7() {
    clear
    echo "\n--> Step 7: Create kernel and initial RAM disk"
    if [ -z "$SCRIPT_OPTION" ]; then
        print_config
        prompt_yn "Create kernel and initial RAM disk (y/n)? "
    else
        echo
        ANS="y"
    fi
    if [ $ANS = "y" ]; then
        script_run "make_images.sh" $SCRIPT_OPTION
    else
        echo "\nSkipped creating kernel and initial RAM disk"
        prompt_enter
    fi
}

step8() {
    if [ -z "$SCRIPT_OPTION" ]; then
        clear
        echo "\n--> Step 8: Install kernel and initial RAM disk on target device"
        print_config
    fi
    if [ "$DEVICE_TYPE" != "all" ]; then
        if [ -z "$SCRIPT_OPTION" ]; then
            prompt_yn "Install kernel and initial RAM disk on target device (y/n)? "
        else
            echo "\n--> Step 8: Install kernel and initial RAM disk on target device\n"
            ANS="y"
        fi
        if [ $ANS = "y" ]; then
            script_run "install_images.sh" $SCRIPT_OPTION
        else
            echo "\nSkipped installing kernel and initial RAM disk"
            prompt_enter
        fi
    else
        if [ -z "$SCRIPT_OPTION" ]; then
            cat <<END

You have indicated that you have a newer version of U-Boot
which supports Flattened Device Tree (FDT) kernels.
As a result, the kernel and initial RAM disk can be run
directly from the /boot directory of new root file system
and do not have to be copied to the reserved space before
the swap partition.
END
            prompt_enter
        else
            echo "\n--> Step 8: Skipped installing kernel and initial RAM disk."
            echo "New style U-Boot specified via DEVICE_TYPE=all.\n"
            echo "Kernel and RAM disk exist in \"/boot\" directory.\n"
        fi
    fi
}

step9() {
    local CMD
    local CWD="$PWD"
    local CWD2
    local DIR
    local COPY_LIST="bin  boot  dev  etc  home  lib  opt  root  sbin  usr  var"
    local INFO="\n--> Step 9: Copy the root file system to target device"
    local MAKE_LIST="media  mnt  proc  run  srv  sys  tmp"
    local MOUNT="rootfs"
    local SOURCE_DIR=$(readlink -f ../..)

    ERROR=0
    if [ -z "$SCRIPT_OPTION" ]; then
        clear
        echo "$INFO"
        print_config
        cat <<END

This step will mount the root file system on device "$DEV_ROOT"
using the following mount point (which will be created temporarily):

    $SOURCE_DIR/$MOUNT

The source file system is:

    $SOURCE_DIR

The following directories will be created via shallow copy (no contents):

    $MAKE_LIST

The following directories will be fully copied:

    $COPY_LIST

The target file system will then customized for the target device.
END
        prompt_yn "Copy files (y/n)? "
        if [ "$ANS" = "n" ]; then
            echo "\nSkipped copying files"
            return
        fi
    fi
    #
    # Select the customizer from within the "tools" directory.
    # Then change to the source root partition.
    #
    select_custom
    cd ../..
    if [ -z "$SCRIPT_OPTION" ]; then
        clear
    fi
    if [ -e $MOUNT ]; then
        echo "\nFile "$SOURCE_DIR/$MOUNT" already exists, removing."
        CMD="rmdir $MOUNT"
        if $CMD; then
            :
        else
            script_error $? "$CMD"
            quit
        fi
    fi
    #
    # TARGET_ROOT is needed by the custom script, below.
    # Create mount directory 
    #
    export TARGET_ROOT="$SOURCE_DIR/$MOUNT"
    echo "\nCreating mount point directory "$TARGET_ROOT""
    CMD="mkdir $MOUNT"
    if $CMD; then
        :
    else
        script_error $? "$CMD"
        quit
    fi
    #
    # Mount device
    #
    echo "\nMounting $DEV_ROOT on $MOUNT"
    CMD="mount $DEV_ROOT $MOUNT"
    if $CMD; then
        :
    else
        script_error $? "$CMD"
        echo "\nRemoving mount point $MOUNT"
        rmdir $MOUNT
        quit
    fi
    #
    # From this point forward, the mount point
    # must be unmounting before exit is called.
    #

    #
    # Shallow copy
    #
    CMD="ls -d $MAKE_LIST | cpio -pdm $MOUNT"
    echo "\nMaking a shallow copy with the following command:\n\n\t$CMD"
    # Need eval due to pipe
    if eval $CMD; then 
        #
        # Deep copy
        #
        CMD="cp -a $COPY_LIST $MOUNT"
        echo "\nMaking a deep copy with the following command:\n\n\t$CMD"
        if $CMD; then 
            :
        else
            script_error $? "$CMD"
        fi
    else
        script_error $? "$CMD"
    fi
    #
    # Run customizer
    #
    if [ $ERROR -eq 0 ]; then
        CWD2="$PWD"
        DIR="boot/tools/custom/$CUSTOM"
        echo "\nChanging directory to \"$DIR\"\nand running \"sh install.sh\"\n"
        CMD="cd $DIR"
        if $CMD; then
            CMD="sh install.sh"
            if $CMD; then
                :
            else
                script_error $? "$CMD"
            fi
            cd "$CWD2"  # return to directory
        else
            script_error $? "$CMD"
        fi
    fi
    #
    # Sync disks
    # 
    cat <<END

Calling sync (this may take a while, particularly with a USB stick).

It is extremely important that the sync completes so that the target
device can be cleanly unmounted by this script.
END
    sync; sync
    #
    # Unmount file system
    # 
    echo "\nUnmounting $MOUNT"
    CMD="umount $MOUNT"
    if $CMD; then
        :
    else
        script_error $? "$CMD"
    fi
    #
    # Remove mount point
    # 
    echo "\nRemoving mount point $MOUNT"
    CMD="rmdir $MOUNT"
    if $CMD; then
        :
    else
        script_error $? "$CMD"
    fi
    if [ $ERROR -ne 0 ]; then
        echo "\nExiting due to error..."
        exit 1
    fi
    cd "$CWD"   # return to boot/tools directory
    echo "\nSuccessfully copied root file system"
}

#
# Validates $ROOT_SIZE, setting it to "" if not -1 or 2-512.
# If 2-512, makes a reasonable try at making sure the choosen
# size does not exceed the size of the device (found in $SECTORS).
#
validate_root_size() {
    local DEV_GIB
    local DEV_MIB
    local SIZE

    if [ "X$ROOT_SIZE" != "X-1" ]; then # -1 is OK
        ROOT_SIZE=$(echo "$ROOT_SIZE" | grep '^[1-9][0-9]*')
        if [ -n "$ROOT_SIZE" ]; then
            if [ ${#ROOT_SIZE} -le 3 -a $ROOT_SIZE -ge 2 -a $ROOT_SIZE -le 512 ]; then
                #
                # Calculate the total size required in sectors.
                # The maximum Max value includes swap (GiB), root (GiB),
                # 1 Mib for the partition table and 64 MiB for the reserved space
                # for the kernel and initial RAM disk for old-style U-Boots.
                #
                # Because this script has to be able to run on a Kirkwood device
                # (which has 32 bit shell integers, thus 2 GiB max int or a 1TB max
                # disk size), regular shell integer math won't work for multi-terrabyte
                # sized disks. As a result, the size required is calculated using integers
                # and this value is compared to the disk size using string comparison.
                # This means comparing string length and then comparing strings.
                #
                SIZE=$(expr $SWAP_SIZE \* 2048 + $ROOT_SIZE \* 2097152 + 133120)
                if [ ${#SECTORS} -gt ${#SIZE} ]; then
                    return # OK
                elif [ ${#SECTORS} -eq ${#SIZE} -a $SECTORS -ge $SIZE ]; then
                    return # OK
                fi
                DEV_GIB=$(expr $SECTORS / 2097152)
                DEV_MIB=$(expr $SECTORS / 2048)
                SIZE=$(expr $SECTORS - $SWAP_SIZE \* 2048 - 133120)
                SIZE=$(expr $SIZE / 2097152)
                echo "\nTarget device is $DEV_GIB GiB (actually, $DEV_MIB MiB)."
                echo "The maximum root size is $SIZE GiB. Either reduce your swap,"
                echo "choose a value between 2 and $SIZE, or use -1 (entire device)."
                ROOT_SIZE=""
                return
            fi
            ROOT_SIZE=""
        fi
        echo "\nRoot size must be a value between 2 and 512."
    fi
}

#
# Check command line arguments
#
for ARG in $@; do
    if [ "$ARG" = "-b" ]; then
        BUILD_OPTION="$ARG"
        if [ -z "$IMAGE_FILE" ]; then
            echo "\ninstall.sh: passed -b option but IMAGE_FILE environment variable is not set."
            exit 1
        fi
    elif [ "$ARG" = "-s" ]; then
        SCRIPT_OPTION="$ARG"
    fi
done

#
# Run main()
#
main
exit 0
