#!/bin/sh
#
# Written by Geoff Mottram (geoff at minaret dot biz).
#
# Placed in the public domain by the author.
#
# See LICENSE.txt for NO WARRANTY statement.
#

#
# This script will set the IX2 U-Boot environment variables
# to support booting the Debian Kirkwood kernel. This script
# must be run on the IX2 using the stock O/S.
#
# If you have console access to the IX2 via a serial cable, the
# following commands can be run instead:
# 
# setenv mem_kernel 0x1000000
# setenv mem_initrd 0x2000000
# setenv off_kernel 0x800
# setenv off_initrd 0x8800
# setenv sz_kernel 0x4000
# setenv sz_initrd 0x6000
# 
# setenv conf_ide 'ide reset; setenv boot_dev ide; setenv bootargs $(console) root=LABEL=rootfs $(bootargs_mtd) earlyprintk=serial $(bootargs_end)'
# setenv conf_usb 'usb reset; setenv boot_dev usb; setenv bootargs $(console) root=LABEL=usbroot $(bootargs_mtd) earlyprintk=serial $(bootargs_end)'
# setenv load_kernel '$(boot_dev) read $(mem_kernel) $(off_kernel) $(sz_kernel)'
# setenv load_initrd '$(boot_dev) read $(mem_initrd) $(off_initrd) $(sz_initrd)'
# setenv boot_mem 'bootm $(mem_kernel) $(mem_initrd)'
# setenv boot_try 'run load_kernel load_initrd boot_mem'
# 
# With USB boot support:
# setenv bootcmd 'echo *Try USB*; run conf_usb boot_try; echo *Try IDE*; run conf_ide boot_try; echo *Try IDE1*; ide device 1; run boot_try'
# 
# Without USB boot support:
# setenv bootcmd 'echo *Try IDE*; run conf_ide boot_try; echo *Try IDE1*; ide device 1; run boot_try'
# 
# Remember to save your environment in U-Boot with the following command:
# 
#     saveenv

#
# For the IX2-ng, the original bootcmd was (without the line breaks):
# 
# bootcmd=ide reset; ide read 0x40000 0x800 0x1800; ide read 0x900000 0x2000 0x1800;
# setenv bootargs $(console) $(bootargs_root) $(bootargs_mtd) $(bootargs_end);
# bootm 0x40000 0x900000;run bootcmd2
#

#
# Configure the script.
#
config() {
    local CMD
    local NAME

    #
    # source functions
    #
    for NAME in "common.sh"; do
        NAME="functions/$NAME"
        if [ -f $NAME ]; then
            . "$NAME"
        else
            echo "\nCannot locate script file \"$NAME\". Exiting...\n"
            exit 1
        fi
    done
    for NAME in "fw_printenv" "fw_setenv"; do
        CMD=$(which fw_printenv)
        if [ -z "$CMD" ]; then
            echo "The program \"$NAME\" is not installed"
            quit
        fi
    done  
    if [ fw_printenv > /dev/null ]; then
        :
    else
        echo "\nCannot run \"fw_printenv\""
        quit
    fi
}

help() {
    clear
    cat <<END

This script will set the IX2 U-Boot environment variables
to support booting the Debian Kirkwood kernel. This script
must be run on the IX2 using the stock O/S and will use the
"fw_setenv" program to set these values.

This script has two steps. You will be given one last chance
to cancel-out in step three just before any changes are made
to your system.

If you have console access to the IX2 via a serial cable, you can
perform all modification with a single flash write (via saveenv).
See the comments at the top of this sript for the appropriate
U-Boot commands.

With the exception of "bootcmd", none of the environment variables
being set are used by the stock O/S. This means you can revert
back to the stock U-Boot command by restoring the original
value of "bootcmd" without having to undo any of the other changes.
END
	prompt_enter
	clear
	cat <<END

It is a very good idea to print your original U-Boot environment
to a file and copy that file to another computer.

    fw_printenv > orig_uboot_env.txt

In the comments at the top of this script there is also a copy of
the original IX2-ng bootcmd, in case you need it.

The version of fw_setenv included on the stock IX2 O/S does not support
script files (a list of environment variables to set all at the same
time). As a result, fw_setenv is called multiple times but only if the
U-Boot environment variable is not already set to the desired value.
END
	prompt_enter
}

#
# Display introduction screen.
#
intro() {
    clear
    cat <<END

This script will set the IX2 U-Boot environment variables
to support booting the Debian Kirkwood kernel. This script
must be run on the IX2 using the stock O/S and will use the
"fw_setenv" program to set these values.

This script has two steps. You will be given one last chance
to cancel-out in step three just before any changes are made
to your system.
END
    prompt_yn "Do you want to see the instructions (y/n)? "
    if [ "$ANS" = "y" ]; then
        help
    fi
}

#
# Main function, runs all steps.
#
main() {
    # Configure script
    config
    # Display intro screen
    intro
    # Run all two steps
    step1
    step2
    echo "\nU-Boot environment update complete.\n"
}

#
# Sets USB_BOOT to either "y" or "n".
#
step1() {
    clear
    cat <<END

--> Step 1: Choose USB or IDE Configuration

Do you want your IX2 to try to boot from a USB stick first,
before trying the IDE drives?
END
    prompt_yn "Install USB boot (y/n)? "
    USB_BOOT="$ANS"
}

#
# Step 2
#
# Set U-Boot environment variables.
#
step2() {
    clear
    cat <<END

--> Step 2: Set U-Boot Environment Variables

Enable USB BOOT: $USB_BOOT

The U-Boot environment variables will now be set based on the
above settings. Each variable will only be changed if it is not
already set to the required value.
END
    prompt_yn "Last chance. Set U-Boot environment (y/n)? "
	echo
    uboot_env 'mem_kernel' '0x1000000'
    uboot_env 'mem_initrd' '0x2000000'
    uboot_env 'off_kernel' '0x800'
    uboot_env 'off_initrd' '0x8800'
    uboot_env 'sz_kernel' '0x4000'
    uboot_env 'sz_initrd' '0x6000'
    uboot_env 'conf_ide' 'ide reset; setenv boot_dev ide; setenv bootargs $(console) root=LABEL=rootfs earlyprintk=serial'
    uboot_env 'conf_usb' 'usb reset; setenv boot_dev usb; setenv bootargs $(console) root=LABEL=usbroot earlyprintk=serial'
    uboot_env 'load_kernel' '$(boot_dev) read $(mem_kernel) $(off_kernel) $(sz_kernel)'
    uboot_env 'load_initrd' '$(boot_dev) read $(mem_initrd) $(off_initrd) $(sz_initrd)'
    uboot_env 'boot_mem' 'bootm $(mem_kernel) $(mem_initrd)'
    uboot_env 'boot_try' 'run load_kernel load_initrd boot_mem'

    if [ "$USB_BOOT" = 'y' ]; then
        uboot_env 'bootcmd' 'echo *Try USB*; run conf_usb boot_try; echo *Try IDE*; run conf_ide boot_try; echo *Try IDE1*; ide device 1; run boot_try'
    else 
        uboot_env 'bootcmd' 'echo *Try IDE*; run conf_ide boot_try; echo *Try IDE1*; ide device 1; run boot_try'
    fi
}

#
# Checks U-Boot environment variable for a certain value and if
# it is not correct, sets the value.
#
uboot_env() {
    local CURRENT
    local EXPECT

    EXPECT="$1=$2"
    CURRENT=$(fw_printenv "$1")
    if [ "$CURRENT" = "$EXPECT" ]; then
        echo "U-Boot variable \"$1\" is already set correctly, skipping."
    else 
        echo "Setting U-Boot variable \"$1\""
        if fw_setenv "$1" "$2"; then
            :
        else
            echo "fw_setenv returned $?. Exiting..."
            exit 1
        fi
    fi
}

#
# Run script
#
main

exit 0
